package com.sooka.component.beetl.fun;

import com.sooka.common.utils.StrUtil;
import com.sooka.module.web.cms.service.CategoryService;
import com.sooka.module.web.system.service.RoleService;
import com.sooka.mybatis.model.TCmsCategory;
import org.beetl.core.Context;
import org.beetl.core.Function;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author limeng
 * @date 2021年10月29日 17:45
 *
 * 用于角色管理，在角色权限分配页面展示和回显当前角色拥有的栏目树
 */
@Service
public class ContentRoleCategoryFunction  implements Function {

    @Autowired
    private CategoryService service;

    @Autowired
    private RoleService roleService;

    @Value("${system.http.protocol}")
    private String httpProtocol;

    private final String isChecked = " data-checked='true' ";

    @Override
    public Object call(Object[] objects, Context context) {
        /*父节点Id*/
        Integer pid = (Integer) objects[0];
        /*角色Id*/
        Integer roleId = (Integer) objects[1];
        /*站点Id*/
        Integer siteId = (Integer) objects[2];
        return recursion(pid, roleId, siteId);
    }

    /* 递归函数 */
    private String recursion(Integer pid,Integer roleId,Integer siteId){
        StringBuffer sbf = new StringBuffer();
        List<TCmsCategory> cats  = service.findCategoryListByPid(Long.parseLong(pid.toString()),siteId);
        if(cats!=null&&cats.size()>0){
            for(TCmsCategory cat:cats){
                int childSize = service.findCategoryListByPid(cat.getCategoryId(),siteId).size();
                if((cat.getAlone()||!StrUtil.isBlank(cat.getUrl()))&&childSize==0){
                    continue;
                }
                sbf.append("  <li data-id=\"" + cat.getCategoryId() + "\" data-pid=\"" + pid + "\" " +
                        isChecked(Integer.parseInt(cat.getCategoryId().toString()),roleId) + " > " + cat.getCategoryName() + " [" + cat.getCategoryId() + "] </li>");
                sbf.append(recursion(Integer.parseInt(cat.getCategoryId().toString()),roleId,siteId));
            }
            return  sbf.toString();
        }
        return "";
    }

    /**
     * 判断是否用于权限
     * @param categoryId
     * @param roleId
     * @return
     */
    public String isChecked(Integer categoryId,Integer roleId){
        if(roleService.findCategoryCountByRoleId(roleId,categoryId)>0)
            return isChecked;
        return "";
    }
}
