package com.sooka.component.beetl.fun;

import com.sooka.common.utils.ControllerUtil;
import com.sooka.common.utils.StrUtil;
import com.sooka.module.web.cms.service.CategoryService;
import com.sooka.module.web.system.service.RoleService;
import com.sooka.mybatis.model.TCmsCategory;
import com.sooka.mybatis.model.TSysRole;
import org.beetl.core.Context;
import org.beetl.core.Function;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author limeng
 * @date 2021年10月30日 9:27
 *
 * 用于内容管理页面，在页面输出带权限隔离的栏目树
 */
@Service
public class ContentTreeRoleCategoryFunction  implements Function {

    @Autowired
    private CategoryService service;

    @Autowired
    private RoleService roleService;

    @Value("${system.http.protocol}")
    private String httpProtocol;

    private String url = "/system/cms/category/input?id";

    @Override
    public Object call(Object[] objects, Context context) {

        /*父节点Id*/
        Integer pid = (Integer) objects[0];
        /*用户Id*/
        Integer userId = (Integer) objects[1];
        /*站点Id*/
        Integer siteId = (Integer) objects[2];

        if(objects[3]!=null&&!StrUtil.isBlank(objects[3].toString()))
            this.url=objects[3].toString();

        return recursion(pid, userId, siteId);
    }

    /* 递归函数 */
    private String recursion(Integer pid,Integer userId,Integer siteId){
        StringBuffer sbf = new StringBuffer();
        List<TCmsCategory> cats  = service.findCategoryListByPid(Long.parseLong(pid.toString()),siteId);
        List<TSysRole> roles = roleService.findByUserId(userId);
        if(cats!=null&&cats.size()>0){
            for(TCmsCategory cat:cats){
                int childSize = service.findCategoryListByPid(cat.getCategoryId(),siteId).size();
                if((cat.getAlone()||!StrUtil.isBlank(cat.getUrl()))&&childSize==0){
                    continue;
                }
                for(TSysRole role : roles){
                    if(hasThisCategory(Integer.parseInt(cat.getCategoryId().toString()),role.getRoleId())){
                        if(childSize>0) {
                            sbf.append("  <li data-id=\"" + cat.getCategoryId() + "\" data-pid=\"" + pid + "\" >" + cat.getCategoryName() + " [" + cat.getCategoryId() + "] </li>");
                        }else {
                            sbf.append("  <li data-id=\"" + cat.getCategoryId() + "\" data-pid=\"" + pid + "\" data-url=\"" + httpProtocol+"://"+ ControllerUtil.getDomain()+url + "=" + cat.getCategoryId() + "\" data-divid=\"#layout-content\">" + cat.getCategoryName() + " [" + cat.getCategoryId() + "] </li>");
                        }
                        sbf.append(recursion(Integer.parseInt(cat.getCategoryId().toString()),userId,siteId));
                    }
                }

            }
            return  sbf.toString();
        }
        return "";
    }

    /**
     * 判断是否用于权限
     * @param categoryId
     * @param roleId
     * @return
     */
    public boolean hasThisCategory(Integer categoryId,Integer roleId){
        if(roleService.findCategoryCountByRoleId(roleId,categoryId)>0)
            return true;
        return false;
    }
}
