package com.sooka.module.web.cms;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import com.sooka.component.lucene.util.IndexObject;
import org.apache.shiro.authz.UnauthenticatedException;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import com.sooka.mybatis.model.TCmsContentDbt;
import com.sooka.mybatis.model.TCmsContentDbtUser;
import com.sooka.mybatis.model.TCmsZrzc;
import com.github.pagehelper.PageInfo;
import com.sooka.common.constant.CmsConst;
import com.sooka.common.utils.CmsUtil;
import com.sooka.common.utils.ControllerUtil;
import com.sooka.module.web.cms.service.DbtService;
import com.sooka.module.web.cms.service.DbtUserService;
import com.sooka.module.web.cms.vo.TCmsContentDbtUserVo;
import com.sooka.module.web.cms.vo.TCmsContentDbtVo;
import com.sooka.module.web.system.vo.UserVo;


@Controller
@RequestMapping("/dbtUser")
public class DbtUserController {


    @Autowired
    private DbtUserService dbtUserService;

    @Autowired
    private DbtService dbtService;

    @RequiresPermissions("dbt:admin")
    @RequestMapping
    public String index(@RequestParam(value = "pageCurrent",defaultValue = "1") Integer pageNumber,
                        @RequestParam(value = "pageSize",defaultValue = "100") Integer pageSize,
                        TCmsContentDbtUser dbtUser,Integer id, Model model){
        model.addAttribute("dbt",dbtService.findById(id));
        dbtUser.setDbtId(id);
        model.addAttribute("model",dbtUserService.page(pageNumber,pageSize,dbtUser));
        model.addAttribute("pojo",dbtUser);
        return "cms/bdt_cy_list";
    }

    @RequiresPermissions("dbt:admin")
    @RequestMapping("/page")
    public String page(@RequestParam(value = "pageCurrent",defaultValue = "1") Integer pageNumber,
                       @RequestParam(value = "pageSize",defaultValue = "50") Integer pageSize,
                       Integer id,

                       Model model){

        UserVo userVo = ((UserVo) ControllerUtil.getHttpSession().getAttribute(CmsConst.SITE_USER_SESSION_KEY));
        if(CmsUtil.isNullOrEmpty(userVo)) {
            throw  new UnauthenticatedException();
        }
        TCmsContentDbtUserVo dbt = new TCmsContentDbtUserVo();
        dbt.setDbtId(id);
        model.addAttribute("model",dbtUserService.page(pageNumber,pageSize,dbt));
        model.addAttribute("dbt",dbtService.findById(id));
        model.addAttribute("pojo",dbt);
        return "cms/bdt_cy_list";
    }

    @RequestMapping("/input")
    public String input(@RequestParam(value = "id",required = false) Integer Id,Integer dbtId, Model model) {
        if (Id!=null) {
            model.addAttribute("dbtUser",dbtUserService.findById(Id));
        }
        model.addAttribute("dbt",dbtService.findById(dbtId));
        return "cms/dbtUser_input";
    }

    @RequestMapping("/dbtUserList")
    @ResponseBody
    public Object dbtUserList(@RequestParam(value = "dbtId",required = false)Integer dbtId, Model model) {
        List<TCmsContentDbtUser> list = new ArrayList<TCmsContentDbtUser>();
        if (dbtId!=null) {
            TCmsContentDbtUser dbtuser = new TCmsContentDbtUser();
            dbtuser.setDbtId(dbtId);
            list= dbtUserService.findList(dbtuser);
        }
        return list;
    }
    /**
     * 	分页搜索
     * @param keyword 名字搜索
     * @return
     */
    @RequestMapping("/dbtUserListForSeach")
    public Object dbtUserListForSeach(@RequestParam(value = "pageCurrent",defaultValue = "1") Integer pageNumber,
                                      @RequestParam(value = "pageSize",defaultValue = "10") Integer pageSize,
            @RequestParam(value = "keyword",required = false)String keyword, Model model) {
        PageInfo p = dbtUserService.pageSer(pageNumber,pageSize,keyword);
        List<IndexObject> list;
        if(p!=null){
            list = p.getList();
            model.addAttribute("model",list);
            model.addAttribute("total",p.getTotal());//总数
            model.addAttribute("pagesize",p.getPageSize());//页数
            model.addAttribute("pagenum",p.getPageNum());
            model.addAttribute("keyword",keyword);
        }else {
            list = new ArrayList<>();
            model.addAttribute("model",list);
            model.addAttribute("total",1);//总数
            model.addAttribute("pagesize",1);//页数
            model.addAttribute("pagenum",1);
            model.addAttribute("keyword",keyword);
        }
        if(ControllerUtil.isMobile()){
            return "www/jlrd-mobile/jlrd_dbxxcx_jg";
        }

        return "www/jlrd/jlrd_dbxxcx_jg";

    }

    @RequestMapping("/save")
    @ResponseBody
    public String save(TCmsContentDbtUser pojo,Model model) {
        if(pojo.getId()!=null) {
            return dbtUserService.update(pojo);
        }
        return dbtUserService.save(pojo);
    }

    @RequestMapping("/delete")
    @ResponseBody
    public String delete(@RequestParam(value = "ids",required = false)Integer[] ids) {
        return dbtUserService.delete(ids);
    }
    //分页查询
    @RequestMapping("/selectDbxx")
    public String search(@RequestParam(value = "pageCurrent",defaultValue = "1") Integer pageNumber,
                         @RequestParam(value = "pageSize",defaultValue = "10") Integer pageSize, Integer dbtType,Model model){
        TCmsContentDbtUser dbtUser = new TCmsContentDbtUser();
        dbtUser.setDbtType(dbtType);
        PageInfo p = dbtUserService.page(pageNumber,pageSize,dbtUser);
        List<IndexObject> list;
        if(p!=null){
            list = p.getList();
            model.addAttribute("model",list);
            model.addAttribute("total",p.getTotal());//总数
            model.addAttribute("pagesize",p.getPageSize());//页数
            model.addAttribute("pagenum",p.getPageNum());
            model.addAttribute("dbtType",dbtType);
        }else {
            list = new ArrayList<>();
            model.addAttribute("model",list);
            model.addAttribute("total",1);//总数
            model.addAttribute("pagesize",1);//页数
            model.addAttribute("pagenum",1);
            model.addAttribute("dbtType",dbtType);
        }

        if(ControllerUtil.isMobile()){
            return "www/jlrd-mobile/jlrd_dbxxcx";
        }
        return "www/jlrd/jlrd_dbxxcx";
    }
}
