package com.sooka.module.web.cms;

import java.sql.SQLException;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import org.apache.shiro.authz.UnauthenticatedException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.ui.Model;
import com.sooka.common.constant.CmsConst;
import com.sooka.common.utils.CmsUtil;
import com.sooka.common.utils.ControllerUtil;
import com.sooka.common.utils.UserUtil;
import com.sooka.module.web.cms.service.MailService;
import com.sooka.module.web.cms.service.ModelFiledService;
import com.sooka.module.web.system.vo.UserVo;
import com.sooka.mybatis.mapper.TSysUserMapper;
import com.sooka.mybatis.model.TCmsMail;
import com.sooka.mybatis.model.TCmsModelFiled;
import com.sooka.mybatis.model.TSysUser;

/**
 * 
 * @author Administrator
 * @date 2021-08-31
 */
@Controller
@RequestMapping("/cms/mail")
public class MailController{
	
	@Value("${system.site.modelId}")
    private Integer modelId;

    @Autowired
    private MailService mailService;
    
    @Autowired
    private ModelFiledService modelFiledService;
    
    @Autowired
    private TSysUserMapper sysUserMapper;
    
    /**
	 * 收件箱
	 */
    @RequestMapping("/inbox_list")
    public String inboxList(@RequestParam(value = "pageCurrent",defaultValue = "1") Integer pageNumber,  
    		@RequestParam(value = "pageSize",defaultValue = "20")Integer pageSize, Model model, TCmsMail pojo) {
    	
    	UserVo userVo = ((UserVo) ControllerUtil.getHttpSession().getAttribute(CmsConst.SITE_USER_SESSION_KEY));
        if(CmsUtil.isNullOrEmpty(userVo)) {
        	System.out.println("无登录信息");
            throw new UnauthenticatedException();
        }
        model.addAttribute("model",mailService.pageInbox(pageNumber,pageSize,userVo.getUserId(), pojo));
        return "cms/mail/inbox_list";
    }

	/**
	 * 写信
	 */
	@RequestMapping("/input")
	public String input(@RequestParam(value = "mailId",required = false) Integer mailId, Model model) {
		
		if(mailId != null) {
            model.addAttribute("mail",mailService.findById(mailId));
        }
		
		List<TCmsModelFiled> cmsModelFileds = modelFiledService.findModelFiledListByModelId(modelId);
		List<TSysUser> userList = sysUserMapper.selectAll();
        model.addAttribute("modelFiled",cmsModelFileds);
        model.addAttribute("userList",userList);
        return "cms/mail/mail_input";
	}
	
	/**
	 * 查看信件
	 */
	@RequestMapping("/mailDetail")
	public String mailDetail(@RequestParam(value = "mailId",required = false) Integer mailId, Model model, @RequestParam(value = "flag",required = false) String flag) {
		if(mailId != null) {
	    	model.addAttribute("mail",mailService.findByMailId(mailId, flag));
		}
        return "cms/mail/mail_detail";
	}

	/**
	 * 发送信件
	 */
	@RequestMapping("/save")
	@ResponseBody
	public String save(TCmsMail mail, HttpServletRequest request, @RequestParam(value = "tag",required = false) String[] tags) throws SQLException {
		System.out.println(mail.getUserIds() + "============");
		UserVo userVo = UserUtil.getSysUserVo();
		String content = request.getParameterValues("CONTENT")[0];
		String file = request.getParameterValues("file")[0];
		return mailService.saveList(mail.getUserIds(), userVo, mail, content, file);
		
	}

    /**
	 * 发件箱
	 */
    @RequestMapping("/outbox_list")
    public String outboxList(@RequestParam(value = "pageCurrent",defaultValue = "1") Integer pageNumber,  
    		@RequestParam(value = "pageSize",defaultValue = "20")Integer pageSize, Model model, TCmsMail pojo) {
    	
    	UserVo userVo = ((UserVo) ControllerUtil.getHttpSession().getAttribute(CmsConst.SITE_USER_SESSION_KEY));
        if(CmsUtil.isNullOrEmpty(userVo)) {
        	System.out.println("无登录信息");
            throw new UnauthenticatedException();
        }
        model.addAttribute("model",mailService.pageOutbox(pageNumber,pageSize,userVo.getUserId(), pojo));
    	
        return "cms/mail/outbox_list";
    }
    
	public String index(Integer pageNumber, Integer pageSize, TCmsMail pojo, Model model) {
		return null;
	}

	/**
	 * 删除信件
	 */
	@RequestMapping("/delete")
	@ResponseBody
	public String delete(@RequestParam(value = "ids",required = false) Integer[] ids,@RequestParam(value = "flag",required = false) Integer flag) throws SQLException {
		return mailService.deleteMails(ids,flag);
	}
    
    
}
 