package com.sooka.module.web.cms;


import com.github.pagehelper.PageInfo;
import com.sooka.common.base.BaseController;
import com.sooka.common.utils.StrUtil;
import com.sooka.module.web.cms.service.*;
import com.sooka.mybatis.model.*;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.ModelAndView;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
@RequestMapping("/cms/zrzc")
public class ZrzcController extends BaseController<TCmsZrzc> {

    @Autowired
    private ZrzcService zrzcService;

    @Autowired
    private ContentService contentService;

    @Autowired
    private CategoryService categoryService;

    @Autowired
    private SiteService siteService;

    @Value("${system.site.name}")
    private String siteName;

    @Value("${system.http.protocol}")
    private String httpProtocol;

    @Value("${system.http.host}")
    private String host;

    @Value("${server.port}")
    private String port;

    @Value("${system.site.prefix}")
    private String sitePrefix;


    @RequestMapping("/queryDescription")
    public ModelAndView QueryTitleDescription (String keyword, String siteId, @RequestParam(value = "number",defaultValue = "1") Integer pageNumber){
        ModelAndView modelAndView=new ModelAndView("www/syjcy/listBySouSuo");
        Map<String,Object> map =new HashMap<String,Object>(2);
        map.put("keyword",keyword);
        PageInfo<TCmsContent> tCmsContentPageInfo = contentService.QueryTitleDescription(pageNumber, map);
        modelAndView.addObject("model",tCmsContentPageInfo);
        modelAndView.addObject("frontPath",httpProtocol+"://"+ host+":"+port+"/"+siteName+"/"+sitePrefix);
        modelAndView.addObject("resPath",httpProtocol+"://"+ host+":"+port+"/"+siteName+"/"+"static"+"/"+"www");
        modelAndView.addObject("keyword",keyword);
        modelAndView.addObject("siteId",siteId);
        return modelAndView;
    }

    @RequestMapping("/childQueryDescription")
    public ModelAndView ChildQueryTitleDescription (String keyword, Integer siteId, Long categoryId, @RequestParam(value = "number",defaultValue = "1") Integer pageNumber){
        ModelAndView modelAndView=new ModelAndView("www/syjcy/child_listBySouSuo");
        TCmsCategory category = categoryService.findById(categoryId);
        TCmsSite site = siteService.findById(siteId);
        Map<String,Object> map =new HashMap<String,Object>(2);
        map.put("keyword",keyword);
        map.put("categoryId",categoryId);
        System.out.println(categoryId+"=categoryId");
        PageInfo<TCmsContent> tCmsContentPageInfo = contentService.QueryTitleDescription(pageNumber, map);
        modelAndView.addObject("model",tCmsContentPageInfo);
        modelAndView.addObject("frontPath",httpProtocol+"://"+ host+":"+port+"/"+siteName+"/"+sitePrefix);
        modelAndView.addObject("resPath",httpProtocol+"://"+ host+":"+port+"/"+siteName+"/"+"static"+"/"+"www");
        modelAndView.addObject("keyword",keyword);
        modelAndView.addObject("siteId",siteId);
        modelAndView.addObject("category",category);
        modelAndView.addObject("site",site);
        return modelAndView;
    }


    @RequestMapping("/input")
    public String input(@RequestParam(value = "id",required = false) Integer Id, Model model) {
        if(Id!=null) {
            model.addAttribute("zrzc",zrzcService.findById(Id));
        }
        return "cms/zrzc_input";
    }


    @Override
    @RequestMapping
    public String index(@RequestParam(value = "pageCurrent",defaultValue = "1") Integer pageNumber,
                        @RequestParam(value = "pageSize",defaultValue = "100") Integer pageSize,
                        TCmsZrzc pojo, Model model){
    	try{
    		PageInfo<TCmsZrzc> list=zrzcService.page(pageNumber,pageSize,pojo);
    		model.addAttribute("model",list);
    		model.addAttribute("pojo",pojo);
    	}catch(Exception e){
    		e.printStackTrace();
    	}
        return "cms/zrzc_list";
    }


    @RequestMapping("/save")
    @ResponseBody
    public String save(TCmsZrzc pojo){
        if(pojo.getId()!=null){
            return zrzcService.update(pojo);
        }
        return zrzcService.save(pojo);
    }

    @RequestMapping("/delete")
    @ResponseBody
    public String delete(@RequestParam(value = "ids",required = false) Integer[] ids) throws SQLException {
        return zrzcService.delete(ids);
    }


    @RequestMapping("/selectAll")
    @ResponseBody
    public Object selectAll(){
        List<TCmsZrzc> list= zrzcService.findAll();
        return list;

    }

    @RequestMapping("/selectInfo")
    public String selectInfo( Integer id, Model model){
        TCmsZrzc zrzc = zrzcService.findById(id);
        if("0" == zrzc.getSex() || "0".equals(zrzc.getSex())){
            zrzc.setSex("男");
        }else {
            zrzc.setSex("女");
        }
        model.addAttribute("zrzc",zrzc);
        return "www/jlrd/ldjj";
    }

    @RequestMapping("/selectById")
    public String selectById( Integer id, Model model){
        TCmsZrzc zrzc = zrzcService.findById(id);
        if("0" == zrzc.getSex() || "0".equals(zrzc.getSex())){
            zrzc.setSex("男");
        }else {
            zrzc.setSex("女");
        }
        model.addAttribute("zrzc",zrzc);
        return "www/jlrd-mobile/ldjj";
    }

}
