package com.sooka.module.web.cms.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.sooka.common.exception.CmsException;
import com.sooka.common.exception.SystemException;
import com.sooka.common.utils.CmsUtil;
import com.sooka.common.utils.JsonUtil;
import com.sooka.component.lucene.util.IndexObject;
import com.sooka.module.web.cms.service.*;
import com.sooka.module.web.cms.vo.TCmsEnterpriseHmdVo;
import com.sooka.module.web.cms.vo.TCmsEnterpriseVo;
import com.sooka.module.web.cms.vo.TCmsEnterpriseZdgzmdVo;
import com.sooka.mybatis.mapper.TCmsEnterpriseHmdMapper;
import com.sooka.mybatis.mapper.TCmsEnterpriseMapper;
import com.sooka.mybatis.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;
import java.util.Map;

;

/**
 * Description:内容服务
 *
 *
 * @create 2017-04-18
 **/
@Service
@CacheConfig(cacheNames = "cms-content-cache")
public class EnterpriseHmdServiceImpl implements EnterpriseHmdService {

    @Value("${system.http.protocol}")
    private String httpProtocol;

    @Autowired
    private TCmsEnterpriseHmdMapper enterpriseHmdMapper;

    @Autowired
    private CategoryService categoryService;

    @Autowired
    private LuceneService luceneService;

    @Autowired
    private ModelService modelService;

    @Override
    public String save(TCmsEnterpriseHmd content) {
        content.setUpdatedate(new Date());
        if(enterpriseHmdMapper.insert(content)>0) {
            return JsonUtil.toSUCCESS("操作成功", "", true);
        }
        return JsonUtil.toERROR("操作失败");
    }

    @Override
    public String update(TCmsEnterpriseHmd pojo) {
        return null;
    }

    @Override
    public String delete(Long[] ids) {
        boolean flag_=false;
        if(ids!=null&&ids.length>0) {
            for (Long id : ids){
                TCmsEnterpriseHmd content = enterpriseHmdMapper.selectByPrimaryKey(id);
                if(CmsUtil.isNullOrEmpty(content)) {
                    continue;
                }
                flag_ =  enterpriseHmdMapper.deleteByPrimaryKey(id)>0;
                /*删除lucene索引*/
                IndexObject indexObject = new IndexObject();
                indexObject.setId(id.toString());
                luceneService.delete(indexObject);
            }
        }
        if(flag_) {
            return JsonUtil.toSUCCESS("操作成功","content-tab",false);
        }
         return JsonUtil.toERROR("操作失败");
    }

    @Override
    public PageInfo<TCmsEnterpriseHmd> page(Integer pageNumber, Integer pageSize, TCmsEnterpriseHmd pojo) {
        PageHelper.startPage(pageNumber,pageSize);
        return new PageInfo(enterpriseHmdMapper.selectByCondition(pojo));
    }

    @Override
    public PageInfo<TCmsEnterpriseHmd> page(Integer pageNumber, Integer pageSize) {
        PageHelper.startPage(pageNumber,pageSize);
        return new PageInfo(enterpriseHmdMapper.selectAll());
    }

    @Override
    public TCmsEnterpriseHmd findById(Long id) {
        return enterpriseHmdMapper.selectByPrimaryKey(id);
    }

    @Override
    public List<TCmsEnterpriseHmd> findList(TCmsEnterpriseHmd pojo) {
        return enterpriseHmdMapper.select(pojo);
    }

    @Override
    public List<TCmsEnterpriseHmd> findAll() {
        return enterpriseHmdMapper.selectAll();
    }


    @Override
    public PageInfo<TCmsEnterpriseHmd> page(Integer pageNumber, Integer pageSize, TCmsEnterpriseHmdVo pojo) {
        PageHelper.startPage(pageNumber,pageSize);
        return new PageInfo(enterpriseHmdMapper.selectByCondition(pojo));
    }
    @Cacheable(key = "'find-contentId-'+#p0+'-tableName-'+#p1")
    @Override
    public Map findContentByContentIdAndTableName(Long contentId, String tableName) {
        Map result =enterpriseHmdMapper.selectByContentIdAndTableName(contentId,tableName);
        if(CmsUtil.isNullOrEmpty(result)) {
            throw new CmsException("内容不存在或已删除！");
        }
        return result;
    }

    @Override
    public String recovery(Long[] ids) {
        /*多此一举,哈哈*/
        boolean flag_=false;
        if(ids!=null&&ids.length>0) {
            for (Long id : ids){
                TCmsEnterpriseHmd cmsContent = enterpriseHmdMapper.selectByPrimaryKey(id);
                if(CmsUtil.isNullOrEmpty(cmsContent)) {
                    continue;
                }
              /*  TCmsCategory category = categoryService.findById(cmsContent.getCategoryId());
                if(!CmsUtil.isNullOrEmpty(category)) {
                    if (!StrUtil.isBlank(category.getPermissionKey()) && !SecurityUtils.getSubject().isPermitted(category.getPermissionKey())) {
                        throw new SystemException("对不起，您没有[" + category.getCategoryName() + "]栏目的管理权限！");
                    }
                }
                if(cmsContent.getStatus()<0) {
                    cmsContent.setStatus(1);
                } else {
                    cmsContent.setStatus(-1);
                }*/
                flag_ =  enterpriseHmdMapper.updateByPrimaryKeySelective(cmsContent)>0;
            }
        }
        if(flag_) {
            return JsonUtil.toSUCCESS("操作成功","content-tab",false);
        }
        return JsonUtil.toERROR("操作失败");
    }

    @Override
    public String save(TCmsEnterpriseHmd content, String tableName, Map<String, Object> formParam, String[] tag) throws SQLException {
        return null;
    }

    @Override
    public String update(TCmsEnterpriseHmd content, String tableName, List<TCmsModelFiled> cmsModelFileds, Map<String, Object> formParam, String[] tag) throws SQLException {
        return null;
    }

    @Transactional(transactionManager = "masterTransactionManager",rollbackFor = Exception.class)
    public int SaveModelFiledParam(Map<String, Object> formParam,TCmsEnterpriseHmd content,String tableName,List<TCmsModelFiled> cmsModelFileds) throws SQLException {

        return 0;
    }


    @Cacheable(key = "'find-siteid-'+#p0+'-categoryid-'+#p1+'-orderby-'+#p2+'-pageNumber-'+#p3+'-pageSize-'+#p4+'-hasChild-'+#p5+'-isHot-'+#p6+'-isPic-'+#p7+'-isRecommend-'+#p8")
    @Override
    public PageInfo<Map> findContentListBySiteIdAndCategoryId(Integer siteId,
                                                                      Long categoryId,
                                                                      Integer orderBy,
                                                                      Integer pageNumber,
                                                                      Integer pageSize,
                                                                      Integer hasChild,
                                                                      Integer isHot,
                                                                      Integer isPic,
                                                                      Integer isRecommend) {
        Long[] categoryIds;
        TCmsCategory category = categoryService.findById(categoryId);
        if(CmsUtil.isNullOrEmpty(category)) {
            throw new CmsException("栏目["+categoryId+"]不存在！");
        }
        TCmsModel model = modelService.findById(category.getModelId());
        /*如果包含子类栏目*/
        if(hasChild==1) {
            List<TCmsCategory> cats =findChildCategory(categoryId);
            /*如果子栏目没有内容就查询当前自身*/
            if(CmsUtil.isNullOrEmpty(cats)){
                categoryIds =new Long[]{categoryId};
            }else{
                categoryIds =new Long[cats.size()+1];
                categoryIds[0]=categoryId;
                int i= 1;
                for(TCmsCategory cat :cats){
                    /*判断是否和父栏目内容模型一致*/
                    if(category.getModelId().intValue()!=cat.getModelId().intValue()) {
                        continue;
                    }
                    categoryIds[i]=cat.getCategoryId();
                    i++;
                }
            }
        }else{
            categoryIds =new Long[]{categoryId};
        }
        PageHelper.startPage(pageNumber, pageSize);

            return new PageInfo<>(enterpriseHmdMapper.selectByContentListBySiteIdAndCategoryIds(siteId,categoryIds,orderBy,isHot,isPic,isRecommend,model.getTableName()));

    }

    private List<TCmsCategory> findChildCategory(Long categoryId){
        List<TCmsCategory> result =  Lists.newArrayList();
        TCmsCategory category = new TCmsCategory();
        category.setParentId(categoryId);
        List<TCmsCategory> list =categoryService.findList(category);
        if(CmsUtil.isNullOrEmpty(list)) {
            return null;
        }
        for (TCmsCategory subCat :list){
            result.add(subCat);
            List<TCmsCategory> subList =  findChildCategory(subCat.getCategoryId());
            if(CmsUtil.isNullOrEmpty(subList)) {
                continue;
            }
            result.addAll(subList);
        }
        return result;
    }

    @Cacheable(key = "'find-siteid-'+#p0+'-categoryids-'+#p1+'-orderby-'+#p2+'-size-'+#p3+'-isHot-'+#p4+'-isPic-'+#p5+'-isRecommend-'+#p6")
    @Override
    public PageInfo<Map> findTopicContentListBySiteIdAndCategoryIds(Integer siteId,
                                                                    Long[] categoryIds,
                                                                    Integer orderBy,
                                                                    Integer size,
                                                                    Integer isHot,
                                                                    Integer isPic,
                                                                    Integer isRecommend) {
        PageHelper.startPage(1, size);
        return new PageInfo<>(enterpriseHmdMapper.selectByTopicContentListBySiteIdAndCategoryIds(siteId,categoryIds,orderBy,isHot,isPic,isRecommend));
    }

    @Override
    public PageInfo<Map> page(Integer pageNumber, Integer siteId, Long categoryId) {
        return null;
    }

    @Override
    public PageInfo<TCmsEnterpriseHmd> findContentListByModelFiledValue(int pageNumber, Long categoryId, String tableName, Map<String, Object> param) {
        return null;
    }


    private String findChildIds(Long categoryId){
        String catIds="";
        TCmsCategory category = new TCmsCategory();
        category.setParentId(categoryId);
        List<TCmsCategory> list =categoryService.findList(category);
        if(CmsUtil.isNullOrEmpty(list)) {
            return categoryId.toString();
        }
        for (TCmsCategory subCat :list){
            catIds+=subCat.getCategoryId()+","+findChildIds(subCat.getCategoryId());
        }
        return catIds.substring(0,catIds.length()-1);
    }

    @Async
    @Override
    public void viewUpdate(Long contentId){
        TCmsEnterpriseHmd content = this.findById(contentId);
        enterpriseHmdMapper.updateByPrimaryKeySelective(content);
    }

    @Override
    public String findAllMonthCount() {
        return null;
    }


    @Cacheable(key = "'content-all-count'")
    @Override
    public Integer AllCount() {
        return this.enterpriseHmdMapper.selectCount(new TCmsEnterpriseHmd());
    }

    @Override
    public List<TCmsEnterpriseHmd> findByCategoryId(Long categoryId) {
        return null;
    }


}
