package com.sooka.module.web.cms.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.sooka.common.exception.CmsException;
import com.sooka.common.exception.SystemException;
import com.sooka.common.utils.CmsUtil;
import com.sooka.common.utils.JsonUtil;
import com.sooka.component.lucene.util.IndexObject;
import com.sooka.module.web.cms.service.*;
import com.sooka.module.web.cms.vo.TCmsEnterpriseVo;
import com.sooka.module.web.cms.vo.TCmsEnterpriseXzcfVo;
import com.sooka.mybatis.mapper.TCmsEnterpriseMapper;
import com.sooka.mybatis.mapper.TCmsEnterpriseXzcfMapper;
import com.sooka.mybatis.model.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.sql.DataSource;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;
import java.util.Map;

;

/**
 * Description:内容服务
 *
 *
 * @create 2017-04-18
 **/
@Service
@CacheConfig(cacheNames = "cms-content-cache")
public class EnterpriseXzcfServiceImpl implements EnterpriseXzcfService {

    @Value("${system.http.protocol}")
    private String httpProtocol;

    @Autowired
    private TCmsEnterpriseXzcfMapper enterpriseXzcfMapper;

    @Autowired
    private CategoryService categoryService;

    @Autowired
    private LuceneService luceneService;

    @Autowired
    private ModelService modelService;

    @Override
    public String save(TCmsEnterpriseXzcf content) {
        content.setUpdatedate(new Date());
        if(enterpriseXzcfMapper.insert(content)>0) {
            return JsonUtil.toSUCCESS("操作成功", "", true);
        }
        return JsonUtil.toERROR("操作失败");
    }

    @Override
    public String update(TCmsEnterpriseXzcf pojo) {
        return null;
    }

    @Override
    public String delete(Long[] ids) {
        boolean flag_=false;
        if(ids!=null&&ids.length>0) {
            for (Long id : ids){
                TCmsEnterpriseXzcf content = enterpriseXzcfMapper.selectByPrimaryKey(id);
                if(CmsUtil.isNullOrEmpty(content)) {
                    continue;
                }
               /* TCmsCategory category = categoryService.findById(content.getCategoryId());
                if(!CmsUtil.isNullOrEmpty(category)) {
                    if (!StrUtil.isBlank(category.getPermissionKey()) && !SecurityUtils.getSubject().isPermitted(category.getPermissionKey())) {
                        throw new SystemException("对不起，您没有[" + category.getCategoryName() + "]栏目的管理权限！");
                    }
                }*/
                flag_ =  enterpriseXzcfMapper.deleteByPrimaryKey(id)>0;
                /*删除lucene索引*/
                IndexObject indexObject = new IndexObject();
                indexObject.setId(id.toString());
                luceneService.delete(indexObject);
            }
        }
        if(flag_) {
            return JsonUtil.toSUCCESS("操作成功","content-tab",false);
        }
         return JsonUtil.toERROR("操作失败");
    }

    @Override
    public PageInfo<TCmsEnterpriseXzcf> page(Integer pageNumber, Integer pageSize, TCmsEnterpriseXzcf pojo) {
        PageHelper.startPage(pageNumber,pageSize);
        return new PageInfo(enterpriseXzcfMapper.selectByCondition(pojo));
    }

    @Override
    public PageInfo<TCmsEnterpriseXzcf> page(Integer pageNumber, Integer pageSize) {
        PageHelper.startPage(pageNumber,pageSize);
        return new PageInfo(enterpriseXzcfMapper.selectAll());
    }

    @Override
    public TCmsEnterpriseXzcf findById(Long id) {
        return enterpriseXzcfMapper.selectByPrimaryKey(id);
    }

    @Override
    public List<TCmsEnterpriseXzcf> findList(TCmsEnterpriseXzcf pojo) {
        return enterpriseXzcfMapper.select(pojo);
    }

    @Override
    public List<TCmsEnterpriseXzcf> findAll() {
        return enterpriseXzcfMapper.selectAll();
    }


    @Override
    public PageInfo<TCmsEnterpriseXzcf> page(Integer pageNumber, Integer pageSize, TCmsEnterpriseXzcfVo pojo) {
        PageHelper.startPage(pageNumber,pageSize);
        return new PageInfo(enterpriseXzcfMapper.selectByCondition(pojo));
    }

    @Cacheable(key = "'find-contentId-'+#p0+'-tableName-'+#p1")
    @Override
    public Map findContentByContentIdAndTableName(Long contentId, String tableName) {
        Map result =enterpriseXzcfMapper.selectByContentIdAndTableName(contentId,tableName);
        if(CmsUtil.isNullOrEmpty(result)) {
            throw new CmsException("内容不存在或已删除！");
        }
        return result;
    }

    @Override
    public String recovery(Long[] ids) {
        /*多此一举,哈哈*/
        boolean flag_=false;
        if(ids!=null&&ids.length>0) {
            for (Long id : ids){
                TCmsEnterpriseXzcf cmsContent = enterpriseXzcfMapper.selectByPrimaryKey(id);
                if(CmsUtil.isNullOrEmpty(cmsContent)) {
                    continue;
                }
                flag_ =  enterpriseXzcfMapper.updateByPrimaryKeySelective(cmsContent)>0;
            }
        }
        if(flag_) {
            return JsonUtil.toSUCCESS("操作成功","content-tab",false);
        }
        return JsonUtil.toERROR("操作失败");
    }

    @CacheEvict(cacheNames ="cms-content-cache",allEntries = true)
    @Transactional(transactionManager = "masterTransactionManager",rollbackFor = Exception.class)
    @Override
    public String save(TCmsEnterpriseXzcf content, String tableName, Map<String, Object> formParam, String[] tags) throws SQLException {

        return JsonUtil.toERROR("操作失败");
    }

    @CacheEvict(cacheNames ="cms-content-cache",allEntries = true)
    @Transactional(transactionManager = "masterTransactionManager",rollbackFor = Exception.class)
    @Override
    public String update(TCmsEnterpriseXzcf content, String tableName, List<TCmsModelFiled> cmsModelFileds, Map<String, Object> formParam, String[] tags) throws SQLException {

        return JsonUtil.toERROR("操作失败");
    }

    @Transactional(transactionManager = "masterTransactionManager",rollbackFor = Exception.class)
    public int SaveModelFiledParam(Map<String, Object> formParam,TCmsEnterpriseXzcf content,String tableName,List<TCmsModelFiled> cmsModelFileds) throws SQLException {

        return 0;
    }

    @Override
    public PageInfo<TCmsEnterpriseXzcf> findContentListByModelFiledValue(int pageNumber,Long categoryId, String tableName, Map<String, Object> param) {
        PageHelper.startPage(pageNumber,this.categoryService.findPageSize(categoryId));
        return new PageInfo<>(enterpriseXzcfMapper.selectByTableNameAndMap(tableName,categoryId,param));
    }

    @Cacheable(key = "'find-siteid-'+#p0+'-categoryid-'+#p1+'-orderby-'+#p2+'-pageNumber-'+#p3+'-pageSize-'+#p4+'-hasChild-'+#p5+'-isHot-'+#p6+'-isPic-'+#p7+'-isRecommend-'+#p8")
    @Override
    public PageInfo<Map> findContentListBySiteIdAndCategoryId(Integer siteId,
                                                                      Long categoryId,
                                                                      Integer orderBy,
                                                                      Integer pageNumber,
                                                                      Integer pageSize,
                                                                      Integer hasChild,
                                                                      Integer isHot,
                                                                      Integer isPic,
                                                                      Integer isRecommend) {
        Long[] categoryIds;
        TCmsCategory category = categoryService.findById(categoryId);
        if(CmsUtil.isNullOrEmpty(category)) {
            throw new CmsException("栏目["+categoryId+"]不存在！");
        }
        TCmsModel model = modelService.findById(category.getModelId());
        /*如果包含子类栏目*/
        if(hasChild==1) {
            List<TCmsCategory> cats =findChildCategory(categoryId);
            /*如果子栏目没有内容就查询当前自身*/
            if(CmsUtil.isNullOrEmpty(cats)){
                categoryIds =new Long[]{categoryId};
            }else{
                categoryIds =new Long[cats.size()+1];
                categoryIds[0]=categoryId;
                int i= 1;
                for(TCmsCategory cat :cats){
                    /*判断是否和父栏目内容模型一致*/
                    if(category.getModelId().intValue()!=cat.getModelId().intValue()) {
                        continue;
                    }
                    categoryIds[i]=cat.getCategoryId();
                    i++;
                }
            }
        }else{
            categoryIds =new Long[]{categoryId};
        }
        PageHelper.startPage(pageNumber, pageSize);

            return new PageInfo<>(enterpriseXzcfMapper.selectByContentListBySiteIdAndCategoryIds(siteId,categoryIds,orderBy,isHot,isPic,isRecommend,model.getTableName()));

    }

    private List<TCmsCategory> findChildCategory(Long categoryId){
        List<TCmsCategory> result =  Lists.newArrayList();
        TCmsCategory category = new TCmsCategory();
        category.setParentId(categoryId);
        List<TCmsCategory> list =categoryService.findList(category);
        if(CmsUtil.isNullOrEmpty(list)) {
            return null;
        }
        for (TCmsCategory subCat :list){
            result.add(subCat);
            List<TCmsCategory> subList =  findChildCategory(subCat.getCategoryId());
            if(CmsUtil.isNullOrEmpty(subList)) {
                continue;
            }
            result.addAll(subList);
        }
        return result;
    }

    @Cacheable(key = "'find-siteid-'+#p0+'-categoryids-'+#p1+'-orderby-'+#p2+'-size-'+#p3+'-isHot-'+#p4+'-isPic-'+#p5+'-isRecommend-'+#p6")
    @Override
    public PageInfo<Map> findTopicContentListBySiteIdAndCategoryIds(Integer siteId,
                                                                    Long[] categoryIds,
                                                                    Integer orderBy,
                                                                    Integer size,
                                                                    Integer isHot,
                                                                    Integer isPic,
                                                                    Integer isRecommend) {
        PageHelper.startPage(1, size);
        return new PageInfo<>(enterpriseXzcfMapper.selectByTopicContentListBySiteIdAndCategoryIds(siteId,categoryIds,orderBy,isHot,isPic,isRecommend));
    }

    @Cacheable(key = "'page-pageNumber-'+#p0+'-siteId-'+#p1+'-categoryId-'+#p2")
    @Override
    public PageInfo<Map> page(Integer pageNumber,Integer siteId, Long categoryId) {

        TCmsCategory category = categoryService.findById(categoryId);
        if(CmsUtil.isNullOrEmpty(category)) {
            throw new CmsException("栏目["+categoryId+"]不存在！");
        }
        TCmsModel model = modelService.findById(category.getModelId());

        /* 如果当前栏目有内容就直接返回*/
        PageHelper.startPage(pageNumber,category.getPageSize());
        List<Map> list =enterpriseXzcfMapper.selectByCategoyId(categoryId,siteId,model.getTableName());
        if(!CmsUtil.isNullOrEmpty(list)&&list.size()>0) {
            return new PageInfo<>(list);
        }

        /*
        * 查询当前栏目的子类
        * todo 有必要确定子栏目的内容模型是否必须和父栏目一致,暂时以父栏目的模型为主
        */
        PageHelper.startPage(pageNumber,category.getPageSize());
        return new PageInfo<>(enterpriseXzcfMapper.selectByCategoyParentId(this.findChildIds(category.getCategoryId()),siteId,model.getTableName()));
    }

    private String findChildIds(Long categoryId){
        String catIds="";
        TCmsCategory category = new TCmsCategory();
        category.setParentId(categoryId);
        List<TCmsCategory> list =categoryService.findList(category);
        if(CmsUtil.isNullOrEmpty(list)) {
            return categoryId.toString();
        }
        for (TCmsCategory subCat :list){
            catIds+=subCat.getCategoryId()+","+findChildIds(subCat.getCategoryId());
        }
        return catIds.substring(0,catIds.length()-1);
    }

    @Async
    @Override
    public void viewUpdate(Long contentId){
        TCmsEnterpriseXzcf content = this.findById(contentId);
        enterpriseXzcfMapper.updateByPrimaryKeySelective(content);
    }

    @Override
    public String findAllMonthCount() {
        Map result = this.enterpriseXzcfMapper.selectAllMonthCount();
        if(CmsUtil.isNullOrEmpty(result)) {
            throw new SystemException("统计报表查询失败！");
        }
        String json = "{" +
                "  \"tooltip\": {\n" +
                "    \"trigger\": \"axis\"" +
                "  },\n" +
                "  \"legend\": {\n" +
                "    \"data\": [\n" +
                "      \"内容数量\"\n" +
                "    ]\n" +
                "  },\n" +
                "  \"toolbox\": {\n" +
                "    \"show\": true,\n" +
                "    \"feature\": {\n" +
                "      \"magicType\": {\n" +
                "        \"show\": true,\n" +
                "        \"type\": [\n" +
                "          \"line\",\n" +
                "          \"bar\"\n" +
                "        ]\n" +
                "      },\n" +
                "      \"restore\": {\n" +
                "        \"show\": true\n" +
                "      },\n" +
                "      \"saveAsImage\": {\n" +
                "        \"show\": true\n" +
                "      }\n" +
                "    }\n" +
                "  },\n" +
                "  \"calculable\": true,\n" +
                "  \"xAxis\": [\n" +
                "    {\n" +
                "      \"type\": \"category\",\n" +
                "      \"data\": [\n" +
                "        \"1月\",\n" +
                "        \"2月\",\n" +
                "        \"3月\",\n" +
                "        \"4月\",\n" +
                "        \"5月\",\n" +
                "        \"6月\",\n" +
                "        \"7月\",\n" +
                "        \"8月\",\n" +
                "        \"9月\",\n" +
                "        \"10月\",\n" +
                "        \"11月\",\n" +
                "        \"12月\"\n" +
                "      ]\n" +
                "    }\n" +
                "  ],\n" +
                "  \"yAxis\": [\n" +
                "    {\n" +
                "      \"type\": \"value\",\n" +
                "      \"splitArea\": {\n" +
                "        \"show\": true\n" +
                "      }\n" +
                "    }\n" +
                "  ],\n" +
                "  \"series\": [\n" +
                "    {\n" +
                "      \"name\": \"内容数量\",\n" +
                "      \"type\": \"bar\",\n" +
                "      \"data\": [\n" +
                "        "+result.get("一月份")+",\n" +
                "        "+result.get("二月份")+",\n" +
                "        "+result.get("三月份")+",\n" +
                "        "+result.get("四月份")+",\n" +
                "        "+result.get("五月份")+",\n" +
                "        "+result.get("六月份")+",\n" +
                "        "+result.get("七月份")+",\n" +
                "        "+result.get("八月份")+",\n" +
                "        "+result.get("九月份")+",\n" +
                "        "+result.get("十月份")+",\n" +
                "        "+result.get("十一月份")+",\n" +
                "        "+result.get("十二月份")+"\n" +
                "      ]\n" +
                "    }\n" +
                "  ]\n" +
                "}";
        return json;
    }

    @Cacheable(key = "'content-all-count'")
    @Override
    public Integer AllCount() {
        return this.enterpriseXzcfMapper.selectCount(new TCmsEnterpriseXzcf());
    }

    @Override
    public List<TCmsEnterpriseXzcf> findByCategoryId(Long categoryId) {
        TCmsEnterpriseXzcf content = new TCmsEnterpriseXzcf();
        return this.enterpriseXzcfMapper.select(content);
    }

    @Override
    public List<TCmsEnterpriseXzcf> selectXzcf() {
        return this.enterpriseXzcfMapper.selectXzcf();
    }

    @Override
    public List<TCmsEnterpriseXzcf> selectByEnterpriseId(Long enterpriseId) {
        return this.enterpriseXzcfMapper.selectByEnterpriseId(enterpriseId);
    }
}
